import rshanker779_common as utils
import pytest
import requests as r


def test_format_header_string():
    headers = """
    GET / HTTP/1.1
    Host: www.bbc.co.uk
    User-Agent: Mozilla/5.0 (X11; Ubuntu; Linux x86_64; rv:74.0) Gecko/20100101 Firefox/74.0
    Accept: text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,*/*;q=0.8
    Accept-Language: en-GB,en;q=0.5
    Accept-Encoding: gzip, deflate, br
    Connection: keep-alive
    Cookie: ckns_explicit=1; ckns_sa_labels_persist={}; ckns_privacy=july2019; ckns_id=eyJkbiI6InJvdmluZ3JvdmVyIiwiYWIiOiJvMTgiLCJwYSI6IkNNOCIsImVwIjp0cnVlLCJldiI6dHJ1ZSwicGwiOmZhbHNlLCJwcyI6InhFV3dTbFRfTHJrNDVOMm9nWnpYckRXSVFCaWEweWFMcS0xYzR3V2x4U3ciLCJjbCI6ZmFsc2UsInNlcy1leHAiOjE1NzA3MTUyODIwMDAsImp3dC1leHAiOjE2MzM3ODYzODIwMDAsInJ0a24tZXhwIjoxNjMzNzg2MzgyMDAwLCJ0a24tZXhwIjoxNTg2MTY4MTU1MDAwfQ; ckns_stateless=1; ckns_sylphid=Den1nwf59VxWE4IjrVZSoX4wYOaHmkKfNAV6aCwpE0g; atuserid=%7B%22name%22%3A%22atuserid%22%2C%22val%22%3A%22b79fb0f0-9d9e-4442-b1e4-248a9e2aa82a%22%2C%22options%22%3A%7B%22end%22%3A%222020-12-06T20%3A34%3A03.465Z%22%2C%22path%22%3A%22%2F%22%7D%7D; ckns_policy=000; ckns_policy_exp=1611081053599; ckns_pgAgeConfirm=true; ckns_bbcTvVod=true; ckns_mvt=d482285d-5a61-4a1e-bee1-84046473edac; ckns_orb_fig_cache={%22ad%22:0%2C%22ap%22:0%2C%22ck%22:1%2C%22eu%22:1%2C%22uk%22:1}; ecos.dt=1586133182864; ckns_nonce=_6-EAPVmOLMjF7WutnoHZt6u; ckns_atkn=eyJ0eXAiOiJKV1QiLCJ6aXAiOiJOT05FIiwia2lkIjoiRWdVVSs4TmhhQVlLYzZ5bG9DQnJuS1BUY2U4PSIsImFsZyI6IkVTMjU2In0.eyJzdWIiOiIxMjk0Njc3ODQ5Njc3MDM4MjkiLCJjdHMiOiJPQVVUSDJfU1RBVEVMRVNTX0dSQU5UIiwiYXV0aF9sZXZlbCI6MCwiYXVkaXRUcmFja2luZ0lkIjoiMTlhMDFmOWMtNDY3Yi00ZWQ5LWFhZWUtMTkxNjVjY2ExNTg2LTM5MTgzNDkzIiwiaXNzIjoiaHR0cHM6Ly9hY2Nlc3MuYXBpLmJiYy5jb20vYmJjaWR2NS9vYXV0aDIiLCJ0b2tlbk5hbWUiOiJhY2Nlc3NfdG9rZW4iLCJ0b2tlbl90eXBlIjoiQmVhcmVyIiwiYXV0aEdyYW50SWQiOiJZV1QzY3h1SnVLR1FRZmVrNl9IY2R1V1pkNzAiLCJhdWQiOiJBY2NvdW50IiwibmJmIjoxNTg2MTY0NTU1LCJncmFudF90eXBlIjoicmVmcmVzaF90b2tlbiIsInNjb3BlIjpbImV4cGxpY2l0IiwidWlkIiwiaW1wbGljaXQiLCJwaWkiLCJjb3JlIiwib3BlbmlkIl0sImF1dGhfdGltZSI6MTU3MDcxNDM4MiwicmVhbG0iOiIvIiwiZXhwIjoxNTg2MTcxNzU1LCJpYXQiOjE1ODYxNjQ1NTUsImV4cGlyZXNfaW4iOjcyMDAsImp0aSI6InZoMXIxcTN5UmZ2RHR1Q1lrTkhlTzFablljOCJ9.s2_cjPjRtHIprcEQ9CrY913Sa_fFgMws9W7gvbHBEmf6IHJeVuE4v_z8Vj6y-vBjrxmp34YEwND8YzMR5f2Hyw; ckns_idtkn=eyJ0eXAiOiJKV1QiLCJraWQiOiJIa2d0WDBJd3RDOStSVGQvOWdYdFN0bk9VaU09IiwiYWxnIjoiUlMyNTYifQ.eyJhdF9oYXNoIjoiNjNjLWRzZ3F4U1FmSC1hNzA0OGlVZyIsInN1YiI6IjEyOTQ2Nzc4NDk2NzcwMzgyOSIsInVzZXJBdHRyaWJ1dGVzIjp7ImFiIjoibzE4IiwiZG4iOiJyb3Zpbmdyb3ZlciIsInBhIjoiQ004IiwiZXAiOnRydWUsImV2Ijp0cnVlLCJwbCI6ZmFsc2UsInBzIjoieEVXd1NsVF9Mcms0NU4yb2daelhyRFdJUUJpYTB5YUxxLTFjNHdXbHhTdyIsInN5IjoiRGVuMW53ZjU5VnhXRTRJanJWWlNvWDR3WU9hSG1rS2ZOQVY2YUN3cEUwZyIsImNsIjpmYWxzZX0sImF1ZGl0VHJhY2tpbmdJZCI6IjE5YTAxZjljLTQ2N2ItNGVkOS1hYWVlLTE5MTY1Y2NhMTU4Ni0zOTE4MzQ5NyIsImlzcyI6Imh0dHBzOi8vYWNjZXNzLmFwaS5iYmMuY29tL2JiY2lkdjUvb2F1dGgyIiwidG9rZW5OYW1lIjoiaWRfdG9rZW4iLCJhdWQiOiJBY2NvdW50IiwiYWNyIjoiMCIsImF6cCI6IkFjY291bnQiLCJhdXRoX3RpbWUiOjE1NzA3MTQzODIsInJlYWxtIjoiLyIsImV4cCI6MTU4NjE2ODE1NSwidG9rZW5UeXBlIjoiSldUVG9rZW4iLCJpYXQiOjE1ODYxNjQ1NTV9.ek1S4OJVXjlbAJFY6ux90JlZpQtHFeBpDlfjF22GzDH0HBlf3heNapUo8BhFIJXxGALt1U8WOmWIh_-Zd59KzFZdiyctP-LyfnVLefqo5CnIMdl6VydSM6JLCweZPQyaLDeRxNpDJDc3mzLLz4A5XNgzozuhGiuYySXCbF2Sta2YiY4Ng0_4UV-Li2fUTeYNDQC-dJvi5YsPC4aTPAWDO-VrHwVqaiyxx0xpMhB2O9ZPeTsJZq2ZXY3owvmpsVkFkhtEuNMFHn1SiusLxC8cdAzOwt2Ly256SCkQrnjdgJPkwlFLpgPP-QnKpx-HreYqUlX_CEAGkZpb7ZGSq_lItQ
    Upgrade-Insecure-Requests: 1
    DNT: 1
    If-None-Match: W/"402f7-0Hc3CLUUujBhc3udatbw53n67J4"
    """
    expected = {
        "Host": "www.bbc.co.uk",
        "User-Agent": "Mozilla/5.0 (X11; Ubuntu; Linux x86_64; rv:74.0) Gecko/20100101 Firefox/74.0",
        "Accept": "text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,*/*;q=0.8",
        "Accept-Language": "en-GB,en;q=0.5",
        "Accept-Encoding": "gzip, deflate, br",
        "Connection": "keep-alive",
        "Cookie": "ckns_explicit=1; ckns_sa_labels_persist={}; ckns_privacy=july2019; ckns_id=eyJkbiI6InJvdmluZ3JvdmVyIiwiYWIiOiJvMTgiLCJwYSI6IkNNOCIsImVwIjp0cnVlLCJldiI6dHJ1ZSwicGwiOmZhbHNlLCJwcyI6InhFV3dTbFRfTHJrNDVOMm9nWnpYckRXSVFCaWEweWFMcS0xYzR3V2x4U3ciLCJjbCI6ZmFsc2UsInNlcy1leHAiOjE1NzA3MTUyODIwMDAsImp3dC1leHAiOjE2MzM3ODYzODIwMDAsInJ0a24tZXhwIjoxNjMzNzg2MzgyMDAwLCJ0a24tZXhwIjoxNTg2MTY4MTU1MDAwfQ; ckns_stateless=1; ckns_sylphid=Den1nwf59VxWE4IjrVZSoX4wYOaHmkKfNAV6aCwpE0g; atuserid=%7B%22name%22%3A%22atuserid%22%2C%22val%22%3A%22b79fb0f0-9d9e-4442-b1e4-248a9e2aa82a%22%2C%22options%22%3A%7B%22end%22%3A%222020-12-06T20%3A34%3A03.465Z%22%2C%22path%22%3A%22%2F%22%7D%7D; ckns_policy=000; ckns_policy_exp=1611081053599; ckns_pgAgeConfirm=true; ckns_bbcTvVod=true; ckns_mvt=d482285d-5a61-4a1e-bee1-84046473edac; ckns_orb_fig_cache={%22ad%22:0%2C%22ap%22:0%2C%22ck%22:1%2C%22eu%22:1%2C%22uk%22:1}; ecos.dt=1586133182864; ckns_nonce=_6-EAPVmOLMjF7WutnoHZt6u; ckns_atkn=eyJ0eXAiOiJKV1QiLCJ6aXAiOiJOT05FIiwia2lkIjoiRWdVVSs4TmhhQVlLYzZ5bG9DQnJuS1BUY2U4PSIsImFsZyI6IkVTMjU2In0.eyJzdWIiOiIxMjk0Njc3ODQ5Njc3MDM4MjkiLCJjdHMiOiJPQVVUSDJfU1RBVEVMRVNTX0dSQU5UIiwiYXV0aF9sZXZlbCI6MCwiYXVkaXRUcmFja2luZ0lkIjoiMTlhMDFmOWMtNDY3Yi00ZWQ5LWFhZWUtMTkxNjVjY2ExNTg2LTM5MTgzNDkzIiwiaXNzIjoiaHR0cHM6Ly9hY2Nlc3MuYXBpLmJiYy5jb20vYmJjaWR2NS9vYXV0aDIiLCJ0b2tlbk5hbWUiOiJhY2Nlc3NfdG9rZW4iLCJ0b2tlbl90eXBlIjoiQmVhcmVyIiwiYXV0aEdyYW50SWQiOiJZV1QzY3h1SnVLR1FRZmVrNl9IY2R1V1pkNzAiLCJhdWQiOiJBY2NvdW50IiwibmJmIjoxNTg2MTY0NTU1LCJncmFudF90eXBlIjoicmVmcmVzaF90b2tlbiIsInNjb3BlIjpbImV4cGxpY2l0IiwidWlkIiwiaW1wbGljaXQiLCJwaWkiLCJjb3JlIiwib3BlbmlkIl0sImF1dGhfdGltZSI6MTU3MDcxNDM4MiwicmVhbG0iOiIvIiwiZXhwIjoxNTg2MTcxNzU1LCJpYXQiOjE1ODYxNjQ1NTUsImV4cGlyZXNfaW4iOjcyMDAsImp0aSI6InZoMXIxcTN5UmZ2RHR1Q1lrTkhlTzFablljOCJ9.s2_cjPjRtHIprcEQ9CrY913Sa_fFgMws9W7gvbHBEmf6IHJeVuE4v_z8Vj6y-vBjrxmp34YEwND8YzMR5f2Hyw; ckns_idtkn=eyJ0eXAiOiJKV1QiLCJraWQiOiJIa2d0WDBJd3RDOStSVGQvOWdYdFN0bk9VaU09IiwiYWxnIjoiUlMyNTYifQ.eyJhdF9oYXNoIjoiNjNjLWRzZ3F4U1FmSC1hNzA0OGlVZyIsInN1YiI6IjEyOTQ2Nzc4NDk2NzcwMzgyOSIsInVzZXJBdHRyaWJ1dGVzIjp7ImFiIjoibzE4IiwiZG4iOiJyb3Zpbmdyb3ZlciIsInBhIjoiQ004IiwiZXAiOnRydWUsImV2Ijp0cnVlLCJwbCI6ZmFsc2UsInBzIjoieEVXd1NsVF9Mcms0NU4yb2daelhyRFdJUUJpYTB5YUxxLTFjNHdXbHhTdyIsInN5IjoiRGVuMW53ZjU5VnhXRTRJanJWWlNvWDR3WU9hSG1rS2ZOQVY2YUN3cEUwZyIsImNsIjpmYWxzZX0sImF1ZGl0VHJhY2tpbmdJZCI6IjE5YTAxZjljLTQ2N2ItNGVkOS1hYWVlLTE5MTY1Y2NhMTU4Ni0zOTE4MzQ5NyIsImlzcyI6Imh0dHBzOi8vYWNjZXNzLmFwaS5iYmMuY29tL2JiY2lkdjUvb2F1dGgyIiwidG9rZW5OYW1lIjoiaWRfdG9rZW4iLCJhdWQiOiJBY2NvdW50IiwiYWNyIjoiMCIsImF6cCI6IkFjY291bnQiLCJhdXRoX3RpbWUiOjE1NzA3MTQzODIsInJlYWxtIjoiLyIsImV4cCI6MTU4NjE2ODE1NSwidG9rZW5UeXBlIjoiSldUVG9rZW4iLCJpYXQiOjE1ODYxNjQ1NTV9.ek1S4OJVXjlbAJFY6ux90JlZpQtHFeBpDlfjF22GzDH0HBlf3heNapUo8BhFIJXxGALt1U8WOmWIh_-Zd59KzFZdiyctP-LyfnVLefqo5CnIMdl6VydSM6JLCweZPQyaLDeRxNpDJDc3mzLLz4A5XNgzozuhGiuYySXCbF2Sta2YiY4Ng0_4UV-Li2fUTeYNDQC-dJvi5YsPC4aTPAWDO-VrHwVqaiyxx0xpMhB2O9ZPeTsJZq2ZXY3owvmpsVkFkhtEuNMFHn1SiusLxC8cdAzOwt2Ly256SCkQrnjdgJPkwlFLpgPP-QnKpx-HreYqUlX_CEAGkZpb7ZGSq_lItQ",
        "Upgrade-Insecure-Requests": "1",
        "DNT": "1",
        "If-None-Match": 'W/"402f7-0Hc3CLUUujBhc3udatbw53n67J4"',
    }
    assert utils.format_header_string(headers) == expected


def test_session():
    s = utils.get_requests_session()
    with pytest.raises(r.exceptions.RequestException) as e:
        s.get("http://bbc.co.uk/made_up_page")
    assert "404" in str(e.value)



import datetime
import logging
import time

import rshanker779_common as utils


@utils.time_it
def example_function():
    return 1


@utils.catch_errors
def error_function():
    raise ValueError


@utils.retry(number_retries=2, time_retry=datetime.timedelta(milliseconds=10))
def error_function_2():
    print(1)
    raise ValueError


@utils.retry
def error_function_3():
    print(1)
    raise ValueError


def test_time_it(caplog):
    with caplog.at_level(logging.INFO):
        res = example_function()
    assert res == 1
    assert len(caplog.messages) == 1
    assert "example_function executed in" in caplog.messages[0]


def test_catch_errors(caplog):
    with caplog.at_level(logging.ERROR):
        res = error_function()
    assert res is None
    assert len(caplog.messages) == 1
    assert "Error calling error_function" in caplog.messages[0]


def test_retry(capsys, caplog):
    start = time.time()
    with caplog.at_level(logging.ERROR):
        res = error_function_2()
    assert res is None
    time_taken = time.time() - start
    out, err = capsys.readouterr()
    assert_retry_logic(caplog, out, time_taken, 2, 0.002, "error_function_2")


def test_default_retry(capsys, caplog):
    start = time.time()
    with caplog.at_level(logging.ERROR):
        res = error_function_3()
    assert res is None
    time_taken = time.time() - start
    out, err = capsys.readouterr()
    assert_retry_logic(caplog, out, time_taken, 3, 0, "error_function_3")


def assert_retry_logic(
    caplog, out, time_taken, expected_count, minimum_time, function_name
):
    assert out.count("1") == expected_count
    assert time_taken > minimum_time
    assert len(caplog.messages) == expected_count
    assert caplog.messages[0] == caplog.messages[1]
    assert f"Error calling {function_name}" in caplog.messages[0]



import rshanker779_common as utils


def test_system_hostname():
    assert isinstance(utils.System.HOSTNAME, str)


def test_system_username():
    assert isinstance(utils.System.USERNAME, str)


def test_platform():
    assert isinstance(utils.System.PLATFORM, str)



import json


class JSONMixin:
    def to_json(self):
        return json.dumps(self.to_dict())

    def to_dict(self):
        return self.__convert_item(self.__dict__)

    def __convert_item(self, value):
        if isinstance(value, JSONMixin):
            return {value.__class__.__name__: value.to_dict()}

        elif isinstance(value, dict):
            return {i: self.__convert_item(v) for i, v in value.items()}
        elif isinstance(value, (list, tuple, set)):
            return [self.__convert_item(i) for i in value]
        return value



import pathlib

import pytest

import rshanker779_common as utils


@pytest.fixture
def example_function():
    def example_string_function(x, y):
        return str(reversed(x)) + y

    return example_string_function


def test_profiler(example_function):
    profiler = utils.Profiler(
        example_function, num_iterations=100, x="xfsjkfljkslf", y="fkdskfl"
    )
    profiler.profile()
    assert pathlib.Path(profiler.filename).is_file()


def test_profiler_estimation(example_function):
    profiler = utils.Profiler(example_function, "afdjkla", "dsajdknsa")
    profiler.profile()
    assert profiler.num_iterations > 100_000
    assert profiler.num_iterations < 300_000



import pytest

import rshanker779_common as utils


@pytest.fixture
def A():
    class A(utils.StringMixin, utils.EqualsMixin, utils.JSONMixin):
        a = 1

        def __init__(self, b):
            self.b = b
            self.d = "a"
            self.e = [1, 2, 3]

        def c(self):
            return 3

    return A



import pathlib

import pytest

import rshanker779_common as utils


@pytest.fixture
def build_init_directories(tmp_path):
    (tmp_path / "setup.py").write_text("import setuptools")
    (tmp_path / "a").mkdir()
    (tmp_path / "a" / "test.py").write_text("")
    (tmp_path / "a" / "b").mkdir()
    (tmp_path / "a" / "b" / "c").mkdir()
    (tmp_path / "a" / "b" / "c" / "nested.py").write_text("")
    yield


def test_init_directories(tmp_path, build_init_directories):
    utils.add_init_files(tmp_path)
    assert not (tmp_path / "__init__.py").is_file()
    assert (tmp_path / "a" / "__init__.py").is_file()
    assert not (tmp_path / "a" / "b" / "__init__.py").is_file()
    assert (tmp_path / "a" / "b" / "c" / "__init__.py").is_file()


def test_relative_path():
    assert utils.relative_path() == pathlib.Path(__file__).parent
    assert utils.relative_path("a", "b") == pathlib.Path(__file__).parent / "a" / "b"


def test_make_dirs(tmp_path):
    res = utils.make_dirs(tmp_path / "example")
    assert (tmp_path / "example").is_dir()
    assert res == (tmp_path / "example")



import abc
import datetime
import logging
import threading
import time
from typing import Hashable, Optional, Set

logger = logging.getLogger(__name__)


class RegularRunnableThread(threading.Thread, abc.ABC):
    _process_map: Set[Hashable] = set()

    def __init__(
        self,
        interval: datetime.timedelta,
        daemon: bool = False,
        error_delay: datetime.timedelta = datetime.timedelta(seconds=5),
    ):
        abc.ABC.__init__(self)
        threading.Thread.__init__(self)

        self.daemon = daemon
        self.interval = interval
        self.last_run_time: Optional[datetime.datetime] = None

        self.error_delay = error_delay

        self.process_id = None
        self.stop = False
        self.start()

    def run(self) -> None:
        while True:
            try:
                if self.stop:
                    break
                now = datetime.datetime.now()
                if (
                    self.last_run_time is None
                    or now - self.last_run_time > self.interval
                ):
                    self.do_task()
                    self.last_run_time = now
            except Exception:
                logger.exception("An error occurred")
                time.sleep(self.error_delay.total_seconds())

    def stop_running(self):
        self.stop = True

    @abc.abstractmethod
    def do_task(self) -> None:
        pass



from rshanker779_common.logger import *
from rshanker779_common.cache import *
from rshanker779_common.decorators import *
from rshanker779_common.filesytem import *
from rshanker779_common.mixins import *
from rshanker779_common.profiler import *
from rshanker779_common.scraping import *
from rshanker779_common.system import *
from rshanker779_common.threads import *



import time

import rshanker779_common as utils


def test_cache():
    def expensive_fun():
        time.sleep(1)
        return 1

    start = time.time()
    cache = utils.Cache(utils.CachingStrategy("res", expensive_fun))
    end = time.time() - start
    assert end < 1
    # Has attr will call the cache, so need to make sure this section is timed

    start = time.time()
    assert hasattr(cache, "res")
    res = cache.res
    end = time.time() - start
    assert res == 1
    assert end > 1
    start = time.time()
    res = cache.res
    assert time.time() - start < 1


def test_multi_cache():
    def expensive_fun():
        time.sleep(0.5)
        return 1

    def expensive_fun_2():
        time.sleep(0.5)
        return 2

    start = time.time()
    cache = utils.Cache(
        utils.CachingStrategy("res1", expensive_fun),
        utils.CachingStrategy("res2", expensive_fun_2),
    )
    end = time.time() - start
    assert end < 0.5
    start = time.time()
    assert hasattr(cache, "res1")
    res1 = cache.res1
    assert res1 == 1
    assert hasattr(cache, "res2")
    res2 = cache.res2
    end = time.time() - start
    assert res2 == 2
    assert end > 1



class B:
    pass


def test_eq(A):
    assert not A(1) == B()
    assert not A(3) == A(5)
    assert A(7) == A(7)


def test_hash(A):
    assert hash(A(4)) == hash(A(4))
    assert hash(A({"a": "b"})) == hash(A({"a": "b"}))
    assert not hash(A(3)) == hash(A(4))



import cProfile
import datetime
import pathlib
import subprocess
import tempfile
from typing import Callable, Optional
import rshanker779_common as utils

logger = utils.get_logger(__name__)


class Profiler:
    def __init__(
        self, function: Callable, *args, num_iterations: Optional[int] = None, **kwargs
    ):
        """
        :param function: Any function
        :param num_iteration: Number of calls to function
        :param args: Any args for the callable
        :param kwargs: Any kwargs for the callable
        """
        self.raw_function = function
        self.num_iterations = num_iterations
        self.filename = None
        if num_iterations is None:
            _, elapsed = utils.function_timer(self.raw_function, *args, **kwargs)
            estimated_iterations = int(1 / elapsed)
            logger.info(f"Running profile with {estimated_iterations} iterations")
            self.num_iterations = estimated_iterations
        def func():
            for _ in range(self.num_iterations):
                function(*args, **kwargs)

        self._func = func
        self.temporary_directory = tempfile.mkdtemp()

    def profile(self):
        filename = "{}_{}.prof".format(
            self.raw_function.__name__,
            datetime.datetime.now().strftime("%y_%m_%d_%H_%M_%S"),
        )
        self.filename = pathlib.Path(self.temporary_directory, filename)

        cProfile.runctx("self._func()", globals(), locals(), self.filename)

    def get_snakeviz_visualisation(self):
        return subprocess.run(["snakeviz", self.filename])  # pragma: no cover



import json

import rshanker779_common as utils


class InnerNested(utils.JSONMixin):
    def __init__(self):
        self.a = 1


class B(utils.JSONMixin):
    def __init__(self):
        self.a = InnerNested()
        self.b = (1, 2, 3)


def test_json_mixin(A):
    res = json.loads(A(3).to_json())
    assert res == {"b": 3, "d": "a", "e": [1, 2, 3]}


def test_nested_classes():
    res = json.loads(B().to_json())
    assert res == {"b": [1, 2, 3], "a": {"InnerNested": {"a": 1}}}



import inspect
import os
import pathlib
from typing import Union

from rshanker779_common import get_logger

logger = get_logger(__name__)


def make_dirs(directory: Union[str, bytes]) -> pathlib.Path:
    directory = pathlib.Path(directory)
    directory.mkdir(parents=True, exist_ok=True)
    return directory


def add_init_files(filepath: Union[str, bytes, pathlib.Path]):
    for root, _, files in os.walk(pathlib.Path(filepath)):
        needs_init_file = any(i for i in files if i.endswith(".py") and i != "setup.py")
        has_init_file = any(i for i in files if i == "__init__.py")
        if needs_init_file and not has_init_file:
            logger.info("Adding init file to directory %s", root)
            pathlib.Path(root, "__init__.py").write_text("")


def relative_path(*args: str) -> pathlib.Path:
    frame = inspect.stack()[1]
    file_path = pathlib.Path(frame.filename)
    return pathlib.Path(file_path.parent, *args)



import logging
from functools import partial
from typing import Callable, Any

logger = logging.getLogger(__name__)


class CachingStrategy:
    def __init__(self, attr_name: str, load_method: Callable[[], Any]):
        """        Holds the information about how to cache an attribute               
        :param attr_name: The name of the attribute that will be created in the class       
        :param load_method: How to retreive the item to be cached"""
        self.attr_name = attr_name
        self.load_method = load_method


class Cache:
    def __init__(self, *strategies: CachingStrategy):
        """Cache implementation for things that are slow to load, e.g. files               
        :param strategies: One or more strategies to implement. Will create an attribute            
            matching the attr_name in the Strategy
        """
        for strategy in strategies:
            attr_name = strategy.attr_name
            hidden_attr_name = "__" + strategy.attr_name
            setattr(self, hidden_attr_name, None)

            def loading_method(
                strategy: CachingStrategy, hidden_attr_name: str, self: Cache
            ) -> Any:
                if getattr(self, hidden_attr_name) is None:
                    loaded_value = strategy.load_method()
                    setattr(self, hidden_attr_name, loaded_value)
                return getattr(self, hidden_attr_name)

            setattr(
                self.__class__,
                attr_name,
                property(partial(loading_method, strategy, hidden_attr_name)),
            )



from typing import Dict
import requests as r


def format_header_string(header_string: str) -> Dict[str, str]:
    header_dict = {}
    for line in header_string.splitlines():
        if ":" not in line:
            continue
        key, value = line.split(":", 1)
        if key.strip():
            header_dict[key.strip()] = value.strip()
    return header_dict


def get_requests_session() -> r.Session:
    session = r.Session()
    session.hooks = {
        "response": lambda response, *args, **kwargs: response.raise_for_status()
    }
    return session



class EqualsMixin:
    def __hash__(self):
        return hash(tuple(sorted(self.__immutable_repr(self.__dict__))))

    def __immutable_repr(self, value):
        if isinstance(value, list):
            return tuple(self.__immutable_repr(i) for i in value)
        if isinstance(value, dict):
            return tuple(
                sorted(
                    (self.__immutable_repr(i), self.__immutable_repr(v))
                    for i, v in value.items()
                )
            )
        return value

    def __eq__(self, other):
        if not isinstance(other, self.__class__):
            return NotImplemented
        return other.__dict__ == self.__dict__



import datetime
import logging

from rshanker779_common.threads import RegularRunnableThread


class ExampleRunnable(RegularRunnableThread):
    def __init__(self):
        super().__init__(datetime.timedelta(seconds=5), daemon=True)

    def do_task(self, *args, **kwargs):
        return print("Time: {}, running".format(datetime.datetime.now()))


class ErrorRunnable(RegularRunnableThread):
    def do_task(self) -> None:
        raise ValueError


def test_thread(capfd):
    r = ExampleRunnable()
    out, err = capfd.readouterr()
    assert "running" in out
    r.stop_running()


def test_error_thread(caplog):
    with caplog.at_level(logging.ERROR):
        r = ErrorRunnable(datetime.timedelta(seconds=1), daemon=True)
    assert "An error occurred" in caplog.messages[0]



from setuptools import setup, find_packages

setup(
    name="rshanker779_common",
    version="0.0.7",
    author="rshanker779",
    author_email="rshanker779@gmail.com",
    description="Common utilities",
    python_requires=">=3.7",
    install_requires=[
        "black",
        "bump2version>=1.0.0",
        "pre-commit~=2.2",
        "pytest>5.4,<6",
        "pytest-cov~=2.8",
        "requests<3",
        "snakeviz",
        "wheel>0.33",
    ],
    packages=find_packages(),
    extras_require={"docs": ["sphinx"]},
)



import time
import datetime
from typing import Callable, Optional, Tuple, Any
from functools import wraps, partial

from rshanker779_common import get_logger

logger = get_logger(__name__)


def function_timer(f: Callable, *args, **kwargs) -> Tuple[Any, float]:
    s = time.perf_counter()
    res = f(*args, **kwargs)
    elapsed = time.perf_counter() - s
    return res, elapsed


def time_it(f: Callable):
    @wraps(f)
    def wrapper_func(*args, **kwargs):
        function_return, elapsed = function_timer(f, *args, **kwargs)
        logger.info(f"{f.__name__} executed in {elapsed:2f} seconds")
        return function_return

    return wrapper_func


class ErrorCatchDecorator:
    def __init__(
        self, number_retries: Optional[int], time_retry: Optional[datetime.timedelta],
    ):
        self.number_retries = number_retries
        self.time_retry = time_retry
        if self.number_retries is None and self.time_retry is None:
            self.number_retries = 3

    def __call__(
        self, f,
    ):
        @wraps(f)
        def wrapped_func(*args, **kwargs):
            for _ in range(self.number_retries):
                try:
                    return f(*args, **kwargs)
                except Exception:
                    logger.exception(
                        f"Error calling {f.__name__} with arguments {args}, {kwargs}"
                    )
                    if self.time_retry is not None:
                        time.sleep(self.time_retry.total_seconds())

        return wrapped_func


def catch_errors(f):
    return ErrorCatchDecorator(1, None)(f)


def retry(f=None, *, number_retries=None, time_retry=None):
    error_decorator = ErrorCatchDecorator(number_retries, time_retry)
    if f is None:
        return error_decorator
    return error_decorator(f)



import logging


def get_logger(name: str, level: int = logging.DEBUG) -> logging.Logger:
    logger = logging.getLogger(name)
    if not logger.handlers:
        logger.addHandler(get_sys_out_handler())
    logger.setLevel(level)
    return logger


def get_default_formatter() -> logging.Formatter:
    return logging.Formatter(
        "%(asctime)s - %(funcName)s - %(levelno)s - %(levelname)s - %(message)s"
    )


def get_sys_out_handler() -> logging.Handler:
    handler = logging.StreamHandler()
    handler.setFormatter(get_default_formatter())
    return handler


def clear_handlers(logger: logging.Logger) -> logging.Logger:
    for handler in logger.handlers[:]:
        handler.flush()
        handler.close()
        logger.removeHandler(handler)
    return logger
